@@ -453,6 +453,11 @@ void MetalRenderPipeline::renderFrame(void* color_texture, int shadowMapSize, co
     }
     
     MTL(beginCommandBuffer());
+    std::unique_ptr<void, void(*)(void *)> guard((void *)1, [](void *) {
+        // Clean up Metal objects in the event of an exception.
+        MTL(endCommandBuffer());
+    });
+
     MTLRenderPassDescriptor* renderpassDesc = [MTLRenderPassDescriptor new];
     if(useTiledPipeline)
     {
@@ -492,8 +497,7 @@ void MetalRenderPipeline::renderFrame(void* color_texture, int shadowMapSize, co
             if (envPart)
             {
                 // Apply rotation to the environment shader.
-                float longitudeOffset = (lightRotation / 360.0f) + 0.5f;
-                envMaterial->modifyUniform("longitude/in2", mx::Value::createValue(longitudeOffset));
+                envMaterial->modifyUniform("envImage/rotation", mx::Value::createValue(lightRotation));
 
                 // Apply light intensity to the environment shader.
                 envMaterial->modifyUniform("envImageAdjusted/in2", mx::Value::createValue(lightHandler->getEnvLightIntensity()));
@@ -542,6 +546,7 @@ void MetalRenderPipeline::renderFrame(void* color_texture, int shadowMapSize, co
         {
             material->getProgram()->bindUniform(mx::HW::ALPHA_THRESHOLD, mx::Value::createValue(0.99f));
         }
+        material->getProgram()->bindTimeAndFrame((float) _timer.elapsedTime(), (float) _frame);
         material->bindViewInformation(viewCamera);
         material->bindLighting(lightHandler, imageHandler, shadowState);
         material->bindImages(imageHandler, _viewer->_searchPath);
@@ -573,6 +578,7 @@ void MetalRenderPipeline::renderFrame(void* color_texture, int shadowMapSize, co
             {
                 material->getProgram()->bindUniform(mx::HW::ALPHA_THRESHOLD, mx::Value::createValue(0.001f));
             }
+            material->getProgram()->bindTimeAndFrame((float) _timer.elapsedTime(), (float) _frame);
             material->bindViewInformation(viewCamera);
             material->bindLighting(lightHandler, imageHandler, shadowState);
             material->bindImages(imageHandler, searchPath);
@@ -640,7 +646,8 @@ void MetalRenderPipeline::renderFrame(void* color_texture, int shadowMapSize, co
             atIndex:0];
         [MTL(renderCmdEncoder) drawPrimitives:MTLPrimitiveTypeTriangle vertexStart:0 vertexCount:3];
     }
-    
+
+    guard.release();
     MTL(endCommandBuffer());
     
     if(captureFrame)
